GOOGLE_SEARCH_IMAGE_GOOGLE_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/1rcowE61X8c832ynh0xOt60TH1rJfcJ6z/view?usp=sharing'
YOUTUBE_SEARCH_IMAGE_GOOGLE_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/11WF7Yjep1IOYPU9TAI0x5zxYMLQWMKyg/view?usp=sharing'